/*
 tlv5637 DAC demo
 Uses the tlv5637 library. For details on the sensor, see: 
 www.ti.com/cn/lit/gpn/tlv5637
 Connections to the Circuit:
 CSB: pin 3
 MOSI: pin 11
 SCK: pin 13
 
 REF:      Apply 2.5 V to get output value from 0 to 5V and use REF_EXTERNAL_TLV5637
       or    Let open  to get output value from 0 to 2V  and use REF_1024MV_TLV5637 
       or    Let open  to get output value from 0 to 4V  and use REF_2048MV_TLV5637 
        
 LED: pin 8 , flashes at the frequency of the generated signal 
 
 OUTA:   plug to a scope to observe a sinus waveform 
 OUTB:   plug to a scope to observe a triangle waveform     
    
 created 14 July 2014
 by Bertrand VANDEPORTAELE
 */
#include <SPI.h>
#include <Tlv5637.h>
 
const int chipSelectPin = 3;  //connect to the Chip Select of the TLV5637
const int AlivePin = 8;      //connect to a LED 
   
#define SIZETAB 300
//corresponds to a period of approximately 625ms


byte sinus[SIZETAB];
byte triangle[SIZETAB]; 
int cpt;

//TLV5637 DAC(chipSelectPin,REF_EXTERNAL_TLV5637);
TLV5637 DAC(chipSelectPin,REF_2048MV_TLV5637);


void setup() {
    DAC.powerOn();
    DAC.speedFast();
    pinMode(AlivePin, OUTPUT);  
    //generate arrays of signal values
    for(int j = 0; j < SIZETAB; j++) 
      {
      sinus[j]=128+127*sin(j*2*3.14/SIZETAB);
      triangle[j]=255.*j/(SIZETAB-1);
      // Serial.print(j); Serial.print(':'); Serial.print(sinus[j]); Serial.println(' ');                 
      }
    cpt=0;     
}
void loop() {
   DAC.writeDACAB(sinus[cpt],triangle[cpt]);
   cpt=(cpt+1)%SIZETAB;
   if (cpt<SIZETAB/2)
     digitalWrite(AlivePin, HIGH);   // set the LED on
   else
     digitalWrite(AlivePin, LOW);   // set the LED off
   delay(2);
} 
 
