/*
 * Copyright (c) 2014 by Bertrand VANDEPORTAELE <bvdp@free.fr>
 * TLV5637 library for arduino.
 *
 * This file is free software; you can redistribute it and/or modify
 * it under the terms of either the GNU General Public License version 2
 * or the GNU Lesser General Public License version 2.1, both as
 * published by the Free Software Foundation.
 */

#include "pins_arduino.h"
#include "Tlv5637.h"

#include <stdio.h>
#include <Arduino.h>
#include <avr/pgmspace.h>


#include <SPI.h>

#define R_DATA_TO_DAC_B_AND_BUFFER_TLV5637 0
#define R_DATA_TO_BUFFER_TLV5637 1
#define R_DATA_TO_DAC_A_AND_DAC_B_BUFFER_TLV5637 2
#define R_CTRL_REG_TLV5637 3

#define PWR_UP_TLV5637 0
#define PWR_DOWN_TLV5637 1

#define SPD_SLOW_TLV5637 0
#define SPD_FAST_TLV5637 1


#define SET_CS(x) PORTD |= (1 << x)
#define CLR_CS(x) PORTD &= ~(1 << x)


TLV5637::TLV5637(uint8_t PIN_CS_init,uint8_t REF_init) {
    SPD=SPD_SLOW_TLV5637;
    PWR=PWR_UP_TLV5637;
    PIN_CS=PIN_CS_init;
    REF=REF_init;


}

void TLV5637::powerOn( ) {
    PWR=PWR_UP_TLV5637;
    /*
    /usr/share/arduino/libraries/SPI
    http://arduino.cc/en/Reference/SPI
    http://arduino.cc/en/Tutorial/SPIIntro
    */

    // start the SPI library and configure it to be compatible with the TLV5637

    SPI.begin();
    SPI.setDataMode(SPI_MODE2);// sclk =HIGH at idle state , falling edge validate the data
    SPI.setBitOrder(MSBFIRST);  // most significant bit first
    SPI.setClockDivider(SPI_CLOCK_DIV4); // 4Mhz clock frequency



    // initalize the chip select pins:
    pinMode(PIN_CS, OUTPUT);
    digitalWrite(PIN_CS, HIGH);


    //set external reference voltage
    setRef(REF);
    writetlv5637(R_DATA_TO_DAC_A_AND_DAC_B_BUFFER_TLV5637,0);

}
void TLV5637::powerOff( ) {
    PWR=PWR_DOWN_TLV5637;
    writetlv5637(R_DATA_TO_DAC_A_AND_DAC_B_BUFFER_TLV5637,0);
}
void TLV5637::speedFast( ) {
    SPD=SPD_FAST_TLV5637;
    writetlv5637(R_DATA_TO_DAC_A_AND_DAC_B_BUFFER_TLV5637,0);
}
void TLV5637::speedSlow( ) {
    SPD=SPD_SLOW_TLV5637;
    writetlv5637(R_DATA_TO_DAC_A_AND_DAC_B_BUFFER_TLV5637,0);
}
void TLV5637::setRef(uint8_t REF_init ) {
    REF=REF_init;
    writetlv5637(R_CTRL_REG_TLV5637,0);
}
void TLV5637::writetlv5637(uint8_t R, uint16_t  Value) {
 // version TLV5626
 //   byte first= (((R&2)<<6) | ((SPD&1)<<6) | ((PWR&1)<<5) | ((R&1)<<4) | (Value & 0xF0)>>4);
 //   byte second=  ( (Value & 0xF) <<4) | (REF & 0x3);
   
    byte first= (((R&2)<<6) | ((SPD&1)<<6) | ((PWR&1)<<5) | ((R&1)<<4) | ( (Value >>6) & 0xF));
    byte second=  ( (Value & 0x3F) <<2) | (REF & 0x3);
    // take the chip select low to select the device:
    CLR_CS(PIN_CS);
    SPI.transfer(first); //Send register location
    SPI.transfer(second);  //Send value to record into register
    // take the chip select high to de-select:
    SET_CS(PIN_CS);
}
void TLV5637::writeDACA(uint16_t Value)
{
    writetlv5637(R_DATA_TO_DAC_A_AND_DAC_B_BUFFER_TLV5637,Value);
}
void TLV5637::writeDACB(uint16_t Value)
{
    writetlv5637(R_DATA_TO_DAC_B_AND_BUFFER_TLV5637,Value);
}
void TLV5637::writeDACAB(uint16_t ValueA,uint16_t ValueB)
{
    writetlv5637(R_DATA_TO_BUFFER_TLV5637,ValueB);
    writetlv5637(R_DATA_TO_DAC_A_AND_DAC_B_BUFFER_TLV5637,ValueA);
}
